/* delayed-operation.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Dummy {

	public class DelayedOperation<T> : Object {
		public weak Set<DelayedOperation> operations {
			private get;
			private construct;
		}

		public uint timeout {
			private get;
			private construct;
		}

		public AsyncOperation<T> op {
			private get;
			private construct;
		}

		public T result {
			private get;
			private construct;
		}

		private uint timeout_source_id;

		construct {
			timeout_source_id = Timeout.add (timeout, callback);
		}

		public DelayedOperation (Set<DelayedOperation> operations,
				AsyncOperation<T> op, T result, uint timeout = 10) {
			this.operations = operations;
			this.op = op;
			this.result = result;
			this.timeout = timeout;
		}

		~DelayedOperation () {
			cancel_timeout ();
		}

		private bool callback () {
			op.completed_with_result (result);

			timeout_source_id = 0;
			operations.remove (this);
			return false;
		}

		private void cancel_timeout () {
			if (timeout_source_id > 0) {
				Source.remove (timeout_source_id);
				timeout_source_id = 0;
			}
		}
	}

}
