/* delayed-search.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Dummy {

	public class DelayedSearch<T> : Object {
		public weak Set<DelayedSearch> searches {
			private get;
			private construct;
		}

		public uint timeout {
			private get;
			private construct;
		}

		public AsyncSearch<T> search {
			private get;
			private construct;
		}

		public Set<T> results {
			private get;
			private construct;
		}

		private uint timeout_source_id;

		construct {
			timeout_source_id = Timeout.add (timeout, callback);
		}

		public DelayedSearch (Set<DelayedSearch> searches, AsyncSearch<T> search,
				Set<T> results, uint timeout = 10) {
			this.searches = searches;
			this.search = search;
			this.results = results;
			this.timeout = timeout;
		}

		~DelayedSearch () {
			cancel_timeout ();
		}

		private bool callback () {
			foreach (T result in results) {
				search.add_hit (result);
			}
			search.finish ();

			timeout_source_id = 0;
			searches.remove (this);
			return false;
		}

		private void cancel_timeout () {
			if (timeout_source_id > 0) {
				Source.remove (timeout_source_id);
				timeout_source_id = 0;
			}
		}
	}

}
