using Gee;

using People.Utils;


namespace People.Backend.Dummy.Test  {

	class DelayedSearch : TestIt.Suite {

		Set<Dummy.DelayedSearch> searches;
		People.Utils.AsyncSearch<string> search;
		uint hits;
		bool done;

		protected override void initialize () {
			name = "Dummy::DelayedSearch";
			add_test_case ("delayed callback", test_delayed_callback);
		}

		protected override void set_up () {
			searches = new HashSet<Dummy.DelayedSearch> ();
			search = new People.Utils.AsyncSearch<string>();

			hits = 0;
			done = false;

			base.set_up ();
		}

		protected override void tear_down () {
			searches = null;
			search = null;

			base.tear_down ();
		}

		private void test_delayed_callback () {
			search.hit_found += hit_found_callback;
			search.done += done_callback;

			var results = new HashSet<string> (str_hash, str_equal);
			results.add ("result");
			var delayed = new Dummy.DelayedSearch<string> (searches, search, results, 100);
			searches.add (delayed);
			search.start();

			async_wait ();

			assert_true (hits == 1);
			assert_true (done == true);
		}

		private void hit_found_callback(People.Utils.AsyncSearch<string> search, string hit) {
			hits++;
			assert_true (hit == "result");
		}

		private void done_callback(People.Utils.AsyncSearch<string> search) {
			done = true;

			async_break ();
		}

		static int main(string[] args) {
			var suite = new DelayedSearch();
			return suite.run();
		}

	}
}
