using Gee;

using People.Utils;


namespace People.Backend.Dummy.Test  {

	class DynamicContactSource : TestIt.Suite {
		private ContactSource source;

		private bool contact_added_callback_called;
		private bool contact_removed_callback_called;
		private Backend.Contact added_contact;
		private Backend.Contact removed_contact;

		protected override void initialize () {
			name = "Dummy::ContactSource[dynamic]";
			add_test_case ("adding unknown contact", test_adding_unknown_contact);
			add_test_case ("adding existing contact", test_adding_existing_contact);
			add_test_case ("removing unknown contact", test_removing_unknown_contact);
			add_test_case ("removing existing contact", test_removing_existing_contact);
		}

		protected override void set_up () {
			var contacts = new HashSet<Backend.Contact> ();

			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("0", fields, contacts, false);
			contacts.add (contact);

			source = new Backend.Dummy.ContactSource (contacts, false);
			source.contact_added += contact_added_callback;
			source.contact_removed += contact_removed_callback;

			contact_added_callback_called = false;
			contact_removed_callback_called = false;
			added_contact = null;
			removed_contact = null;

			base.set_up ();
		}

		protected override void tear_down () {
			source = null;

			added_contact = null;
			removed_contact = null;

			base.tear_down ();
		}

		private void test_adding_unknown_contact () {
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("1", fields, source.contacts, false);

			assert_true (source.contacts.size == 1);
			assert_false (contact_added_callback_called);
			assert_true (source.add_contact (contact));
			assert_true (contact_added_callback_called);
			assert_true (added_contact.id == contact.id);
			assert_true (source.contacts.size == 2);
		}

		private void test_adding_existing_contact () {
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("0", fields, source.contacts, false);

			assert_true (source.contacts.size == 1);
			assert_false (contact_added_callback_called);
			assert_false (source.add_contact (contact));
			assert_false (contact_added_callback_called);
			assert_true (source.contacts.size == 1);
		}

		private void test_removing_unknown_contact () {
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("1", fields, source.contacts, false);

			assert_true (source.contacts.size == 1);
			assert_false (contact_removed_callback_called);
			assert_false (source.remove_contact (contact));
			assert_false (contact_removed_callback_called);
			assert_true (source.contacts.size == 1);
		}

		private void test_removing_existing_contact () {
			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var contact = new Backend.Dummy.Contact ("0", fields, source.contacts, false);

			assert_true (source.contacts.size == 1);
			assert_false (contact_removed_callback_called);
			assert_true (source.remove_contact (contact));
			assert_true (contact_removed_callback_called);
			assert_true (removed_contact.id == contact.id);
			assert_true (source.contacts.size == 0);
		}

		private void contact_added_callback (Backend.ContactSource source,
				Backend.Contact contact) {
			contact_added_callback_called = true;
			added_contact = contact;
		}

		private void contact_removed_callback (Backend.ContactSource source,
				Backend.Contact contact) {
			contact_removed_callback_called = true;
			removed_contact = contact;
		}

		static int main(string[] args) {
			var suite = new DynamicContactSource ();
			return suite.run ();
		}

	}
}
