/* contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Xml;

using People;
using People.Utils;


namespace People.Backend.Friendfeed {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {
		private Map<Message, AsyncSearch> pending_searches =
				new HashMap<Message, AsyncSearch> ();

		private Map<Message, AsyncOperation> pending_operations =
				new HashMap<Message, AsyncOperation> ();

		private Soup.Session session = new Soup.SessionAsync ();

		public string user {
			get;
			construct;
		}

		public ContactSource (string user) {
			this.user = user;
			this.flags = Backend.ContactSourceFlags.NEED_NETWORK;
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id);
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			var operation = async_operation_init_bool ();
			var url = "http://friendfeed.com/api/user/%s/profile?format=xml".printf (id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			session.queue_message (msg, validate_contact_id_callback);

			return operation;
		}

		private void validate_contact_id_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			pending_operations.remove (msg);

			if (msg.status_code == KnownStatusCode.OK && msg.response_body.data != null) {
				async_operation_emit_bool (operation, true);
			} else {
				async_operation_emit_bool (operation, false);
			}
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump() {
			var search = new AsyncSearch<Backend.Contact>();

			var url = "http://friendfeed.com/api/user/%s/profile?format=xml".printf (user);
			var msg = new Soup.Message ("GET", url);
			pending_searches.set (msg, search);
			session.queue_message (msg, dump_callback);

			return search;
		}

		private void dump_callback (Session session, Message msg) {
			var search = pending_searches.get (msg);

			if (msg.status_code == KnownStatusCode.OK) {
				Xml.Doc* doc = Xml.Parser.parse_memory (msg.response_body.data,
						(int) msg.response_body.length);
				if (doc != null && doc->children != null) {
					for (Xml.Node* node = doc->children->children; node != null; node = node->next) {
						if (node->name == "subscription") {
							for (Xml.Node* entry = node->children; entry != null; entry = entry->next) {
								if (entry->name == "nickname") {
									/* friend */
									var contact = get_contact_object (entry->get_content ());
									search.add_hit (contact);
								}
							}
						} else if (node->name == "nickname") {
							/* user */
							var contact = get_contact_object (node->get_content ());
							search.add_hit (contact);
						}
					}
				}

				delete doc;
			}

			search.finish ();
			pending_searches.remove (msg);
		}
		/* end of DumpableContactSource interface */

	}
}
