/* contact-source-factory.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;

using People;
using People.Utils;


namespace People.Backend.Google {

	public class ContactSourceFactory : Backend.BaseContactSourceFactory {

		public override Backend.ContactSourceParamSpec[] get_parameters () {
			var parameters = new Backend.ContactSourceParamSpec[2];
			parameters[0] = new Backend.ContactSourceParamSpec ("account",
				typeof(string), Backend.ContactSourceParamFlags.MANDATORY);
			parameters[1] = new Backend.ContactSourceParamSpec ("password",
				typeof(string), Backend.ContactSourceParamFlags.MANDATORY |
				                ContactSourceParamFlags.HIDDEN);
			return parameters;
		}

		public override AsyncOperation<Backend.ContactSource> get_contact_source (Map<string, string>? parameters) {
			var session = new Soup.SessionSync ();
			var msg = Soup.form_request_new ("POST",
					"https://www.google.com/accounts/ClientLogin",
					"account_type", "HOSTED_OR_GOOGLE",
					"Email", parameters.get ("account"),
					"Passwd", parameters.get ("password"),
					"service", "cp",
					"source", "PeopleProject-GoogleContactSource-1");
			session.send_message (msg);
			weak MessageBody body = msg.response_body;

			string authentication_token = null;
			string[] list = body.data.split ("\n");
			foreach (string e in list) {
				string[] tmp = e.split ("=");
				if (tmp[0] == "Auth")
				{
					authentication_token = tmp[1];
					break;
				}
			}

			return sync_result (new ContactSource(authentication_token));
		}
	}

}
