/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People;
using People.Utils;


namespace People.Backend.Keyfile {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {

		public string file_path {
			get;
			construct;
		}

		private KeyFile key_file;

		construct {
			key_file = new KeyFile();
			DirUtils.create_with_parents (Path.get_dirname (file_path), 0700);
			FileStream.open(file_path, "a"); // ensure that the file exists
			try {
				key_file.load_from_file(file_path,
					KeyFileFlags.KEEP_COMMENTS | KeyFileFlags.KEEP_TRANSLATIONS);
			} catch (KeyFileError e) {
				//key_file = null;
				warning ("Failed to load keyfile: %s", e.message);
			}
			flags |= Backend.ContactSourceFlags.CAN_CREATE_CONTACT;
		}

		public ContactSource (string file_path) {
			this.file_path = file_path;
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			return sync_result_bool (key_file.has_group (id));
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id, file_path, key_file);
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Contact> dump () {
			var contacts = key_file.get_groups ();
			var search = new AsyncSearch<Backend.Contact> ();

			foreach (string id in contacts) {
				var contact = get_contact_object (id);
				search.add_hit (contact);
			}
			search.finish ();

			return search;
		}
		/* end of DumpableContactSource interface */

	}
}
