


using People;


namespace People.Backend.Sqlite.Test {

	class SQLQueryVisitor : TestIt.Suite {
		private Backend.Sqlite.SQLQueryVisitor visitor;

		protected override void initialize () {
			name = "People::Backend::Sqlite::SQLQueryVisitor";
			add_test_case ("select all query", test_select_all_query);
			add_test_case ("complex query", test_complex_query);
		}

		protected override void set_up () {
			visitor = new Backend.Sqlite.SQLQueryVisitor ();
		}

		protected override void tear_down () {
			visitor = null;
		}

		private void test_select_all_query () {
			var sql_query = visitor.translate (null);
			assert_true (sql_query == "SELECT id FROM people;");
		}

		private void test_complex_query () {
			var q0 = new Gee.ArrayList<Backend.Query> ();
			var q1 = new Gee.ArrayList<Backend.Query> ();
			var q2 = new Gee.ArrayList<Backend.Query> ();
			q1.add (new FieldQuery("communication.phone.mobile"));
			q1.add (new ValueQuery("8"));

			q0.add (new CompositeQuery (Backend.QueryOperator.EQUALS, q1));
			q2.add (new FieldQuery ("identity.name.first"));
			q2.add (new ValueQuery ("Ali"));
			q0.add (new CompositeQuery (Backend.QueryOperator.CONTAINS, q2));

			var query = Backend.Query.not (Backend.Query.or (q0));

			var sql_query = visitor.translate (query);
			assert_true (sql_query == "SELECT id FROM people WHERE (NOT ((mobile_phone LIKE \"8\") OR (first_name LIKE \"Ali\")));");
		}

		static int main(string[] args) {
			var suite = new SQLQueryVisitor ();
			return suite.run ();
		}
	}
}
