/* contact-source.vala
 *
 * Copyright (C) 2008 Jiqing Qiang <jiqing.qiang@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;
using Soup;
using Json;

using People;
using People.Utils;


namespace People.Backend.Twitter {

	public class ContactSource : Backend.BaseContactSource,
			Backend.DumpableContactSource {
		private Map<Message, AsyncSearch> pending_searches =
				new HashMap<Message, AsyncSearch> ();

		private Map<Message, AsyncOperation> pending_operations =
				new HashMap<Message, AsyncOperation> ();

		private Soup.Session session;

		public string account {
			get;
			construct;
		}

		public string password {
			private get;
			construct;
		}

		construct {
			session = new Soup.SessionAsync ();
			session.authenticate += authentication_callback;
		}

		public ContactSource (string account, string password) {
			this.account = account;
			this.password = password;
			this.flags = Backend.ContactSourceFlags.NEED_NETWORK;
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new Contact (id, session);
		}

		public override AsyncOperation<bool> validate_contact_id (string id) {
			var operation = async_operation_init_bool ();
			var url = "http://twitter.com/users/show/%s.json".printf(id);
			var msg = new Soup.Message ("GET", url);
			pending_operations.set (msg, operation);
			session.queue_message (msg, validate_contact_id_callback);

			return operation;
		}

		private void validate_contact_id_callback (Session session, Message msg) {
			var operation = pending_operations.get (msg);
			pending_operations.remove (msg);

			if (msg.status_code == KnownStatusCode.OK) {
				async_operation_emit_bool (operation, true);
			} else {
				async_operation_emit_bool (operation, false);
			}
		}

		/* DumpableContactSource interface */
		public AsyncSearch<Backend.Contact> dump() {
			var search = new AsyncSearch<Backend.Contact>();

			var url = "http://twitter.com/statuses/friends/%s.json".printf (account);
			var msg = new Soup.Message ("GET", url);
			pending_searches.set (msg, search);
			session.queue_message (msg, dump_callback);

			return search;
		}

		private void dump_callback (Session session, Message msg) {
			var search = pending_searches.get (msg);

			if (msg.status_code == KnownStatusCode.OK) {
				var parser = new Parser ();
				parser.load_from_data (msg.response_body.data,
						(ulong) msg.response_body.data.size);

				weak Json.Array root = parser.get_root ().get_array ();

				for (uint i = 0; i < root.get_length (); i++) {
					string id = root.get_element (i).get_object ().get_member ("screen_name").get_string ();
					var contact = get_contact_object (id);
					search.add_hit (contact);
				}
			}

			search.finish ();
			pending_searches.remove (msg);
		}
		/* end of DumpableContactSource interface */

		private void authentication_callback (Session session, Message msg, Auth auth, bool retrying) {
			auth.authenticate (account, password);
		}
	}
}
