/* address-book.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;


namespace PeopleService {

	[DBus (name = "net.launchpad.People.AddressBook")]
	public class AddressBook : PeopleService.DBusObject {

		public string id {
			get;
			private set construct;
		}

		[DBus (visible = false)]
		public weak People.AddressBook address_book {
			get;
			private set construct;
		}

		public ContactManager contact_manager;
		public Set<QueryResult> query_results = new HashSet<QueryResult> ();
		private uint last_query_id = 0;

		construct {
			contact_manager = new ContactManager (connection, this);
		}

		public AddressBook (DBus.Connection connection, string id, People.AddressBook abook) {
			this.connection = connection;
			this.object_path = "/net/launchpad/People/AddressBook/" + id;

			this.id = id;
			this.address_book = abook;
		}

		public void query (string query, out string bus_name, out DBus.ObjectPath object_path) {
			var result = address_book.query ();
			var query_result = new QueryResult (connection, last_query_id.to_string (), result, this);
			query_results.add (query_result);

			bus_name = query_result.bus_name;
			object_path = new DBus.ObjectPath (query_result.object_path);
			last_query_id++;
		}

	}

}
