/* query-result.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;


namespace PeopleService {

	[DBus (name = "net.launchpad.People.QueryResult")]
	public class QueryResult : PeopleService.DBusObject {
		public string id {
			get;
			private set construct;
		}

		[DBus (visible = false)]
		public People.QueryResult query_result {
			get;
			private set construct;
		}

		[DBus (visible = false)]
		public weak AddressBook address_book {
			get;
			private set construct;
		}

		public signal void result_added (string result_path);
		public signal void result_removed (string result_path);
		public signal void done ();

		construct {
			query_result.notify["state"] += query_result_state_changed;

			query_result.result_fetched += query_result_result_fetched;
			query_result.result_discarded += query_result_result_discarded;
		}

		public QueryResult (DBus.Connection connection, string id, People.QueryResult query_result, AddressBook address_book) {
			this.connection = connection;
			this.object_path = "/net/launchpad/People/QueryResult/%s/%s".printf (address_book.id, id);

			this.id = id;
			this.query_result = query_result;
			this.address_book = address_book;
		}

		public void start () {
			query_result.fetch ();
		}

		public void stop () {
			//FIXME: stop the search
		}

		public void close () {
			address_book.query_results.remove (this);
		}

		public void get_results (out string[] results) {
			var contacts = query_result.results;
			var contact_manager = address_book.contact_manager;

			if (contacts.size > 0) {
				results = new string[contacts.size];
				uint i = 0;
				foreach (People.Contact contact in contacts) {
					var c = contact_manager.get_service_contact (contact);
					results[i] = c.object_path;
					i++;
				}
			}
		}

		private void query_result_state_changed (People.QueryResult result,
				ParamSpec pspec) {
			if (result.state == People.QueryResultState.FETCHED) {
				done ();
			}
		}

		private void query_result_result_fetched (People.QueryResult result, People.Contact contact) {
			var contact_manager = address_book.contact_manager;
			var c = contact_manager.get_service_contact (contact);
			result_added (c.object_path);
		}

		private void query_result_result_discarded (People.QueryResult result, People.Contact contact) {
			var contact_manager = address_book.contact_manager;
			var c = contact_manager.get_service_contact (contact);
			result_removed (c.object_path);
		}
	}

}
