/* test-contact-source-persistence.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;

using People;
using Core.Async;


namespace People.Backend.Test {

	public abstract class ContactSourcePersistence : Core.Trial.Suite {
		protected string backend_name = "Null";
		protected Backend.ContactSourceFactory contact_source_factory;
		protected Core.Container.HashMap<string, string> parameters;
		protected Backend.ContactSource contact_source;
		protected string contact_id = null;

		protected override void initialize () {
			name = "People::Backend::%s::ContactSource".printf (backend_name);
			add_test_case ("contact creation persistence", test_contact_creation_persistence);
		}

		protected override void set_up () {
			try {
				var async_result = contact_source.request_contact (null);
				var contact = async_result.get();
				var fields = new Core.Container.HashMap<string, string> (str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", "John");
				fields.set ("identity.name.last", "Doe");
				contact.set_fields (fields);

				contact_id = contact.id;
			} catch (Error e) {
				failure ();
			}

			contact_source = null;
		}

		protected override void tear_down () {
			contact_source_factory = null;
			contact_source = null;
		}

		private void async_callback (AsyncResult async_result) {
			async_break ();
		}

		private void test_contact_creation_persistence () {
			try {
				var async_result = contact_source_factory.request_contact_source (parameters);
				async_result.add_callback (async_callback);
				async_wait ();

				contact_source = async_result.get ();
			} catch (Backend.Error e) {
				failure ();
			}

			try {
				var async_result = contact_source.request_contact (contact_id);
				async_result.add_callback (async_callback);
				async_wait ();

				var contact = async_result.get ();
				assert_true (contact != null);
			} catch (Error e) {
				failure ();
			}
		}

	}
}
