/* base-contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;


namespace People.Backend {

	/**
	 * BaseContact is an abstract class implementing {@link Contact}.
	 * <p>
	 * This stub of implementation extends {@link AsyncResultHelper} to
	 * provide methods to deal with {@link AsyncResult} creation and
	 * completion.
	 */
	public abstract class BaseContact : Object, Contact {
		public ContactFlags flags {
			get;
			protected set;
			default = ContactFlags.NONE;
		}

		public string? id {
			get;
			protected set;
			default = null;
		}

		/* Contact interface methods */
		public abstract AsyncResult<Set<string>> get_defined_fields ();

		public abstract AsyncResult<Set<string>> get_definable_fields ();

		public abstract AsyncResult<Set<string>> get_mandatory_fields ();

		public abstract AsyncResult<ContactFieldSpec> get_field_spec (string field_name);

		public abstract AsyncResult<Map<string, string>> get_fields (Set<string> fields);

		public abstract AsyncResult<Set<string>> set_fields (Map<string, string> fields);

		public abstract AsyncResult<bool> @delete ();
		/* end of Contact interface methods */
	}
}
