
using Core.Container;
using Core.Async;

namespace People.Backend.Test {
	class BaseContactStub : Backend.BaseContact {
		public BaseContactStub(string? id) {
			this.id = id;
		}

		public override AsyncResult<Set<string> > get_defined_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Set<string> > get_definable_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Set<string> > get_mandatory_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			return new SimpleAsyncResult<Backend.ContactFieldSpec> ();
		}

		public override AsyncResult<Map<string, string> > get_fields(Set<string> fields) {
			return new SimpleAsyncResult<Map<string, string> > ();
		}

		public override AsyncResult<Set<string> > set_fields(Map<string, string> fields) {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<bool> @delete() {
			return new SimpleAsyncResult<bool> ();
		}

		public void do_set_id(string? id) {
			this.id = id;
		}
	}

	class BaseContactSourceStub : Backend.BaseContactSource {

		public override AsyncResult<bool> validate_contact_id (string id) {
			return new ImmediateResult<bool> (true);
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new BaseContactStub (id);
		}
	}

	class BaseContactSourceFactoryStub : Backend.BaseContactSourceFactory {

		public override Backend.ContactSourceParamSpec[] get_parameters () {
			var parameters = new Backend.ContactSourceParamSpec[2];
			parameters[0] = new Backend.ContactSourceParamSpec ("username",
					typeof(string), Backend.ContactSourceParamFlags.MANDATORY);
			parameters[1] = new Backend.ContactSourceParamSpec ("password",
					typeof(string), Backend.ContactSourceParamFlags.NONE);
			return parameters;
		}

		public override AsyncResult<Backend.ContactSource> get_contact_source (Map<string, string>? parameters) {
			return new ImmediateResult<Backend.ContactSource> (new BaseContactSourceStub ());
		}
	}


	class BaseContactSourceFactory : Core.Trial.Suite {
		private BaseContactSourceFactoryStub factory;

		protected override void initialize () {
			name = "People::Backend::BaseContactSourceFactory";
			add_test_case ("correct parameters", test_correct_parameters);
			add_test_case ("incorrect parameters", test_incorrect_parameters);
			add_test_case ("null parameters", test_null_parameters);
		}

		protected override void set_up () {
			factory = new BaseContactSourceFactoryStub ();
		}

		protected override void tear_down () {
			factory = null;
		}

		private void test_correct_parameters () {
			var args = new HashMap<string, string> (str_hash, str_equal, str_equal);

			args.set ("username", "kimbix");
			args.set ("password", "linox45");

			var async_result = factory.request_contact_source (args);

			try {
				async_result.get ();
				success ();
			} catch (Backend.Error e) {
				failure ();
			}
		}

		private void test_incorrect_parameters () {
			var args = new HashMap<string, string> (str_hash, str_equal, str_equal);
			args.set ("password", "linox45");

			var async_result = factory.request_contact_source (args);

			try {
				async_result.get ();
				failure ();
			} catch (Backend.Error e) {
				success ();
			}
		}

		private void test_null_parameters () {
			var async_result = factory.request_contact_source (null);
			try {
				async_result.get ();
				failure ();
			} catch (Backend.Error e) {
				success ();
			}
		}

		static int main(string[] args) {
			var suite = new BaseContactSourceFactory ();
			return suite.run();
		}
	}
}
