
using Core.Container;
using Core.Async;

namespace People.Backend.Test {
	class BaseContactStub : Backend.BaseContact {
		public BaseContactStub(string? id) {
			this.id = id;
		}

		public override AsyncResult<Set<string> > get_defined_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Set<string> > get_definable_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Set<string> > get_mandatory_fields() {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			return new SimpleAsyncResult<Backend.ContactFieldSpec> ();
		}

		public override AsyncResult<Map<string, string> > get_fields(Core.Container.Set<string> fields) {
			return new SimpleAsyncResult<Map<string, string> > ();
		}

		public override AsyncResult<Set<string> > set_fields(Map<string, string> fields) {
			return new SimpleAsyncResult<Set<string> > ();
		}

		public override AsyncResult<bool> @delete() {
			return new SimpleAsyncResult<bool> ();
		}

		public void do_set_id(string? id) {
			this.id = id;
		}
	}

	class BaseContactSourceStub : Backend.BaseContactSource {
		public override AsyncResult<bool> validate_contact_id (string id) {
			return new ImmediateResult<bool> (true);
		}

		public override Backend.Contact create_contact_object (string? id) {
			return new BaseContactStub (id);
		}
	}

	class BaseContactSource : Core.Trial.Suite {
		private BaseContactSourceStub source;
		private BaseContactStub contact;
		private bool added;
		private bool removed;

		protected override void initialize () {
			name = "People::Backend::BaseContactSource";
			add_test_case ("contact added", test_contact_added);
			add_test_case ("contact removed", test_contact_removed);
		}

		protected override void set_up () {
			source = new BaseContactSourceStub ();
			source.contact_added += contact_added_callback;
			source.contact_removed += contact_removed_callback;

			contact = (BaseContactStub) source.get_contact_object ("dummy-id");

			added = false;
			removed = false;
		}

		protected override void tear_down () {
			source = null;
		}

		private void test_contact_added () {
			contact.do_set_id ("other");
			assert_true (added);
			assert_false (removed);
		}

		private void test_contact_removed () {
			contact.do_set_id (null);
			assert_false (added);
			assert_true (removed);
		}

		private void contact_added_callback(Backend.ContactSource source, Backend.Contact contact) {
			added = true;
		}

		private void contact_removed_callback(Backend.ContactSource source, Backend.Contact contact) {
			removed = true;
		}

		static int main(string[] args) {
			var suite = new BaseContactSource ();
			return suite.run();
		}
	}
}
