/* queryable-contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Core.Container;
using Core.Async;

namespace People.Backend {
	/**
	 * Interface to be implemented by {@link ContactSource} implementations
	 * that support content selection through a {@link Query}.
	 */
	public interface QueryableContactSource : ContactSource {
		/**
		 * Returns all the {@link Contact} matching the given {@link Query}
		 * in this {@link ContactSource}.
		 *
		 * @param query the Query to use as a selection
		 * @return      an {@link IncrementalResult} to be populated by Contact objects
		 */
		public abstract IncrementalResult<Set<Contact>> query (Query? query);
	}
}
