using Core.Container;

using People;


namespace People.Backend.Test {

	class Query : Core.Trial.Suite, Backend.QueryVisitor {
		public uint nb_composite;
		public uint nb_field;
		public uint nb_value;

		protected override void initialize () {
			name = "People::Backend::Query";
			add_test_case ("visitor", test_visitor);
			add_test_case ("NOT structure error", test_structure_error_NOT);
			add_test_case ("AND structure error", test_structure_error_AND);
			add_test_case ("OR structure error", test_structure_error_OR);
			add_test_case ("EQUALS structure error", test_structure_error_EQUALS);
		}

		protected override void set_up () {
			nb_composite = 0;
			nb_field = 0;
			nb_value = 0;
		}

		protected override void tear_down () {
		}

		private void test_visitor () {
			/* (AND (NOT (EQUALS identity.name.first identity.name.last)
					(CONTAINS communication.email.home "gmail.com"))) */
			var q0 = new Core.Container.ArrayList<Backend.Query> ();
			var q1 = new Core.Container.ArrayList<Backend.Query> ();
			var q2 = new Core.Container.ArrayList<Backend.Query> ();

			q1.add (new Backend.FieldQuery("identity.name.first"));
			q1.add (new Backend.FieldQuery("identity.name.last"));
			q0.add (Backend.Query.not (new Backend.CompositeQuery (Backend.QueryOperator.EQUALS, q1)));

			q2.add (new Backend.FieldQuery ("communication.email.home"));
			q2.add (new Backend.ValueQuery ("gmail.com"));
			q0.add (new Backend.CompositeQuery (Backend.QueryOperator.CONTAINS, q2));

			var query = Backend.Query.and (q0);
			query.accept (this);

			assert_true (nb_composite == 4);
			assert_true (nb_field == 3);
			assert_true (nb_value == 1);
		}

		private void test_structure_error_NOT () {
			/* (NOT po.pu.pi) */
			var query = (Backend.CompositeQuery) Backend.Query.not (new Backend.FieldQuery ("po.pu.pi"));
			try {
				query.evaluate (new HashMap<string, string> (str_hash, str_equal, str_equal));
				failure ();
			} catch (Backend.QueryError e) {
				success ();
			}
		}

		private void test_structure_error_AND () {
			/* (AND pi.pu.po "dude") */
			var _q = new ArrayList<Backend.Query> ();
			_q.add (new Backend.FieldQuery ("pi.pu.po"));
			_q.add (new Backend.ValueQuery ("dude"));

			var query = (Backend.CompositeQuery) Backend.Query.and (_q);
			try {
				query.evaluate (new HashMap<string, string> (str_hash, str_equal, str_equal));
				failure ();
			} catch (Backend.QueryError e) {
				success ();
			}
		}

		private void test_structure_error_OR () {
			/* (OR pi.pu.po "dude") */
			var _q = new ArrayList<Backend.Query> ();
			_q.add (new Backend.FieldQuery ("pi.pu.po"));
			_q.add (new Backend.ValueQuery ("dude"));

			var query = (Backend.CompositeQuery) Backend.Query.or (_q);
			try {
				query.evaluate (new HashMap<string, string> (str_hash, str_equal, str_equal));
				failure ();
			} catch (Backend.QueryError e) {
				success ();
			}
		}

		private void test_structure_error_EQUALS () {
			/* (EQUALS (NOT "a") (NOT "b")) */
			var _q = new ArrayList<Backend.Query> ();
			_q.add (Backend.Query.not (new Backend.ValueQuery ("a")));
			_q.add (Backend.Query.not (new Backend.ValueQuery ("b")));
			var query = new Backend.CompositeQuery (Backend.QueryOperator.EQUALS, _q);
			try {
				query.evaluate (new HashMap<string, string> (str_hash, str_equal, str_equal));
				failure ();
			} catch (Backend.QueryError e) {
				success ();
			}
		}


		public void visit_composite (Backend.CompositeQuery query) {
			nb_composite++;
			foreach (Backend.Query subquery in query.queries) {
				subquery.accept (this);
			}
		}

		public void visit_field (Backend.FieldQuery query) {
			nb_field++;
		}

		public void visit_value (Backend.ValueQuery query) {
			nb_value++;
		}

		static int main (string[] args) {
			var suite = new Query ();
			return suite.run ();
		}
	}
}
