/* address-book.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;


namespace People {
	public class AddressBook : Object {
		public BiMap<string, Backend.ContactSource> sources {
			public get;
			private construct set;
		}

		public Gee.Set<Gee.List<string> > merged_contacts {
			public get;
			private construct set;
		}

		private ContactManager contact_manager;
		private QueryResultManager query_manager;

		construct {
			contact_manager = new ContactManager (this, merged_contacts);
			query_manager = new QueryResultManager (this, contact_manager);
		}

		public AddressBook(BiMap<string, Backend.ContactSource> sources,
				Gee.Set<Gee.List<string> > merged_contacts) {
			this.sources = sources;
			this.merged_contacts = merged_contacts;
		}

		/* Public API */
		public QueryResult query (string? query = null) {
			return query_manager.query (query);
		}
		/* End of public API */
	}

}
