/* proxy-contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using Gee;

using People.Utils;

namespace People {

	public class ProxyContact : AsyncOperationHelper, Backend.Contact {
		private static Backend.ContactFieldSpec default_field_spec =
				new Backend.ContactFieldSpec(Backend.ContactFieldFlags.NONE);

		public Backend.ContactFlags flags {
			get { return contact.flags; }
			protected set { contact.flags = value; }
			default = Backend.ContactFlags.NONE;
		}

		public string? id {
			get { return contact.id; }
			protected set { contact.id = value; }
			default = null;
		}

		public Backend.Contact contact {
			private get;
			private construct set;
		}

		public ProxyContact (Backend.Contact contact) {
			this.contact = contact;
		}

		public AsyncOperation<Gee.Set<string> > get_defined_fields() {
			return contact.get_defined_fields ();
		}

		public AsyncOperation<Gee.Set<string> > get_definable_fields() {
			return contact.get_definable_fields ();
		}

		public AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			return contact.get_mandatory_fields ();
		}

		public AsyncOperation<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			return contact.get_field_spec (field_name);
		}

		public AsyncOperation<Gee.Map<string, string> >
				get_fields(Gee.Set<string> fields) {
			return contact.get_fields (fields);
		}

		public AsyncOperation<Gee.Set<string> >
				set_fields(Gee.Map<string, string> fields) {
			return contact.set_fields (fields);
		}

		public AsyncOperation<bool> @delete() {
			return contact.delete ();
		}
	}

}
