
namespace People.Test {

	class AddressBookManager : TestIt.Suite {
		private string file_path = "/tmp/people-test-address-book-manager.xml";
		private People.AddressBookManager manager;

		protected override void initialize () {
			name = "People::AddressBookManager";
			add_test_case ("list address books", test_list_address_books);
			add_test_case ("get address book", test_get_address_book);
		}

		protected override void set_up () {
			var file = FileStream.open(file_path, "w");
			file.puts("<people>\n");
			file.puts("    <contact-sources>\n");
			file.puts("        <contact-source id=\"local-db\" class=\"sqlite\">\n");
			file.puts("            <param name=\"db-path\">/tmp/people-test-address-book-manager.db</param>\n");
			file.puts("        </contact-source>\n");

			file.puts("        <contact-source id=\"local-keyfile\" class=\"keyfile\">\n");
			file.puts("            <param name=\"file-path\">/tmp/people-test-address-book-manager.ini</param>\n");
			file.puts("        </contact-source>\n");
			file.puts("    </contact-sources>\n");

			file.puts("    <address-books>\n");
			file.puts("        <address-book id=\"main\">\n");
			file.puts("            <contact-source id=\"local-db\"/>\n");
			file.puts("            <contact-source id=\"local-keyfile\"/>\n");
			file.puts("        </address-book>\n");
			file.puts("    </address-books>\n");
			file.puts("</people>\n");
			file = null;
			var config = new People.ConfigXml(file_path, Environment.get_tmp_dir ());
			manager = new People.AddressBookManager(config);
		}

		protected override void tear_down () {
			manager = null;
			FileUtils.unlink (file_path);
			FileUtils.unlink ("/tmp/people-test-address-book-manager.db");
			FileUtils.unlink ("/tmp/people-test-address-book-manager.ini");
		}

		private void test_list_address_books() {
			var address_books = manager.list_address_books();
			assert_true (address_books.size == 1);
			assert_true (address_books.contains("main"));
		}

		private void test_get_address_book() {
			var address_book = manager.get_address_book("main");
			assert_true (address_book != null);
		}

		static int main(string[] args) {
			var suite = new AddressBookManager();
			return suite.run();
		}
	}
}
