using Gee;

using People.Utils;


namespace People.Test {

	class Contact : TestIt.Suite {
		private Backend.ContactSource source0;
		private Backend.ContactSource source1;
		private People.AddressBook address_book;

		private AsyncOperation op;

		protected override void initialize () {
			name = "People::Contact";
			add_test_case ("defined fields", test_defined_fields);
			add_test_case ("get fields", test_get_fields);
		}

		protected override void set_up () {
			/* source 0 */
			var contacts0 = new HashSet<Backend.Contact> ();

			var first0 = new string[] {"Ali", "Johann", "Monchieu"};
			var last0 = new string[] {"Sabil", "Prieur", "Baboul"};
			for(int i = 0; i < first0.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first0[i]);
				fields.set ("identity.name.last", last0[i]);
				var c = new Backend.Dummy.Contact ("c0%d".printf (i), fields, contacts0, false);
				contacts0.add (c);
			}

			source0 = new Backend.Dummy.ContactSource (contacts0, false);

			/* source 1 */
			var contacts1 = new HashSet<Backend.Contact> ();

			var first1 = new string[] {"Ali", "Bu", "John"};
			var last1 = new string[] {"Sabil", "Bronze", "Doe"};
			var email1 =  new string[] { "ali.sabil@NOSPAM.com", "spam@NOBU.fi", "johny@stars.us" };
			for(int i = 0; i < first1.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first1[i]);
				fields.set ("identity.name.last", last1[i]);
				fields.set ("communication.email.home", email1[i]);
				var c = new Backend.Dummy.Contact ("c1%d".printf (i), fields, contacts1, true);
				contacts1.add (c);
			}

			source1 = new Backend.Dummy.ContactSource (contacts1, true);

			/* sources */
			var map = new Gee.HashMap<string, Backend.ContactSource> (str_hash, str_equal);
			map.set ("source0", source0);
			map.set ("source1", source1);
			var sources = new BiMap<string, Backend.ContactSource> (map, str_hash, str_equal);

			/* meta contacts definition */
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> (str_equal);
			merge1.add ("source0:c00");
			merge1.add ("source0:c02");
			merge1.add ("source1:c10");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> (str_equal);
			merge2.add ("source0:c01");
			merge2.add ("source1:c11");
			merged_contacts.add (merge2);

			/* AddressBook construction */
			address_book = new People.AddressBook (sources, merged_contacts);
		}

		protected override void tear_down () {
			source0 = null;
			source1 = null;

			address_book = null;
			op = null;
		}

		private void test_defined_fields () {
			QueryResult result = address_book.query ();
			result.result_fetched += defined_fields_result_fetched_callback;

			result.fetch ();
		}

		private void defined_fields_result_fetched_callback (QueryResult result, People.Contact contact) {
			if (contact.id == "source0:c00+source0:c02+source1:c10") {
				op = contact.get_defined_fields ();
				op.add_callback (test_defined_fields_callback);
			}
		}

		private void test_defined_fields_callback(AsyncOperation<Gee.Set<string> > op) {
			var fields = new string[] { "identity.name.first",
										"identity.name.last",
										"communication.email.home" };

			var result = op.get_result ();
			foreach (string field in fields) {
				assert_true (result.contains (field));
			}
		}

		private void test_get_fields () {
			QueryResult result = address_book.query ();
			result.result_fetched += fields_result_fetched_callback;

			result.fetch ();
		}

		private void fields_result_fetched_callback (QueryResult result, People.Contact contact) {
			var fields = new Gee.HashSet<string> (str_hash, str_equal);
			fields.add ("identity.name.first");
			fields.add ("communication.email.home");

			if (contact.id == "source0:c00+source0:c02+source1:c10") {
				op = contact.get_fields (fields);
				op.add_callback (test_get_fields_callback);
			}
		}

		private void test_get_fields_callback (AsyncOperation<Gee.Map<string, string> > op) {
			var result = op.get_result ();

			assert_true (result.size == 2);

			string first = result.get ("identity.name.first");
			assert_true (first != null);
			assert_true ((first == "Ali") || (first == "Monchieu"));

			string second = result.get ("communication.email.home");
			assert_true (second != null);
			assert_true (second == "ali.sabil@NOSPAM.com");
		}

		static int main(string[] args) {
			var suite = new Contact();
			return suite.run();
		}
	}
}
