using Gee;

using People.Utils;


namespace People.Test {

	class MetaContacts : TestIt.Suite {
		private Backend.Dummy.ContactSource source0;
		private Backend.Dummy.ContactSource source1;

		private People.AddressBook address_book;
		private QueryResult result;
		private bool result_fetched_callback_called;
		private bool contact_updated_callback_called;

		protected override void initialize () {
			name = "People::MetaContacts";
			add_test_case ("dynamic existing meta contacts", test_dynamic_existing_meta_contacts);
			add_test_case ("dynamic new meta contacts", test_dynamic_new_meta_contacts);
		}

		protected override void set_up () {
			/* source 0 */
			var contacts0 = new HashSet<Backend.Contact> ();

			var first0 = new string[] { "Ali", "Johann" };
			var last0 = new string[] { "Sabil", "Prieur" };
			for(int i = 0; i < first0.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first0[i]);
				fields.set ("identity.name.last", last0[i]);
				var c = new Backend.Dummy.Contact ("%d".printf (i), fields, contacts0, false);
				contacts0.add (c);
			}

			source0 = new Backend.Dummy.ContactSource (contacts0, false);

			/* source 1 */
			var contacts1 = new HashSet<Backend.Contact> ();

			var first1 = new string[] { "asabil", "jprieur", "ieei" };
			for(int i = 0; i < first1.length; i++) {
				var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
				fields.set ("identity.name.first", first1[i]);
				var c = new Backend.Dummy.Contact ("%d".printf (i), fields, contacts1, true);
				contacts1.add (c);
			}

			source1 = new Backend.Dummy.ContactSource (contacts1, true);

			/* sources */
			var map = new Gee.HashMap<string, Backend.ContactSource> (str_hash, str_equal);
			map.set ("source0", new ProxyContactSource (source0));
			map.set ("source1", new ProxyContactSource (source1));
			var sources = new BiMap<string, Backend.ContactSource> (map, str_hash, str_equal);

			/* meta contacts definition */
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> (str_equal);
			merge1.add ("source0:0");
			merge1.add ("source1:0");
			merge1.add ("source0:2");
			merge1.add ("source1:3");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> (str_equal);
			merge2.add ("source0:1");
			merge2.add ("source1:1");
			merged_contacts.add (merge2);
			var merge3 = new Gee.ArrayList<string> (str_equal);
			merge3.add ("source0:3");
			merge3.add ("source1:4");
			merged_contacts.add (merge3);

			/* AddressBook construction */
			address_book = new People.AddressBook (sources, merged_contacts);

			result_fetched_callback_called = false;
			contact_updated_callback_called = false;
		}

		protected override void tear_down () {
			source0 = null;
			source1 = null;

			address_book = null;
			result = null;
		}

		private void test_dynamic_existing_meta_contacts () {
			result = address_book.query ();
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			result.fetch ();
			async_wait ();

			assert_true (result.results.size == 3);
			assert_true (contact_updated_callback_called);
			contact_updated_callback_called = false;

			foreach (People.Contact contact in result.results) {
				if (contact is People.MultiContact) {
					var contacts = ((People.MultiContact) contact).contacts;
					if (contact.id == "source0:0+source1:0+source0:2+source1:3") {
						assert_true (contacts.get (0) != null);
						assert_true (contacts.get (0).id == "0");
						assert_true (contacts.get (1) != null);
						assert_true (contacts.get (0).id == "0");
						assert_true (contacts.get (2) == null);
						assert_true (contacts.get (3) == null);
					}
				}
			}

			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var source02 = new Backend.Dummy.Contact ("2", fields, source0.contacts, false);
			assert_true (source0.add_contact (source02));

			assert_true (contact_updated_callback_called);
			contact_updated_callback_called = false;
			assert_true (result.results.size == 3);

			foreach (People.Contact contact in result.results) {
				if (contact is People.MultiContact) {
					var contacts = ((People.MultiContact) contact).contacts;
					if (contact.id == "source0:0+source1:0+source0:2+source1:3") {
						assert_true (contacts.get (0) != null);
						assert_true (contacts.get (0).id == "0");
						assert_true (contacts.get (1) != null);
						assert_true (contacts.get (1).id == "0");
						assert_true (contacts.get (2) != null);
						assert_true (contacts.get (2).id == "2");
						assert_true (contacts.get (3) == null);
					}
				}
			}

			var source13 = new Backend.Dummy.Contact ("3", fields, source1.contacts, false);
			assert_true (source1.add_contact (source13));

			assert_true (contact_updated_callback_called);
			contact_updated_callback_called = false;
			assert_true (result.results.size == 3);

			foreach (People.Contact contact in result.results) {
				if (contact is People.MultiContact) {
					var contacts = ((People.MultiContact) contact).contacts;
					if (contact.id == "source0:0+source1:0+source0:2+source1:3") {
						assert_true (contacts.get (0) != null);
						assert_true (contacts.get (0).id == "0");
						assert_true (contacts.get (1) != null);
						assert_true (contacts.get (1).id == "0");
						assert_true (contacts.get (2) != null);
						assert_true (contacts.get (2).id == "2");
						assert_true (contacts.get (3) != null);
						assert_true (contacts.get (3).id == "3");
					}
				}
			}
		}

		private void test_dynamic_new_meta_contacts () {
			result = address_book.query ();
			result.notify["state"] += state_changed_callback;
			result.result_fetched += result_fetched_callback;

			result.fetch ();
			async_wait ();

			assert_true (result.results.size == 3);

			var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			var source03 = new Backend.Dummy.Contact ("3", fields, source0.contacts, false);
			assert_true (source0.add_contact (source03));
			async_wait ();

			assert_true (result_fetched_callback_called);
			assert_true (contact_updated_callback_called);
			result_fetched_callback_called = false;
			contact_updated_callback_called = false;

			assert_true (result.results.size == 4);

			var source14 = new Backend.Dummy.Contact ("4", fields, source1.contacts, false);
			assert_true (source1.add_contact (source14));
			assert_false (result_fetched_callback_called);
			assert_true (contact_updated_callback_called);
			result_fetched_callback_called = false;
			contact_updated_callback_called = false;

			assert_true (result.results.size == 4);
		}

		private void result_fetched_callback (QueryResult result, Contact contact) {
			result_fetched_callback_called = true;
			contact.contact_updated += contact_updated_callback;
			if (result.state == QueryResultState.FETCHED) {
				async_break ();
			}
		}

		private void contact_updated_callback (Contact contact) {
			contact_updated_callback_called = true;
		}

		private void state_changed_callback (QueryResult result, ParamSpec spec) {
			if (result.state == QueryResultState.FETCHED) {
				async_break ();
			}
		}

		static int main(string[] args) {
			var suite = new MetaContacts();
			return suite.run();
		}
	}
}
