using Gee;

using People;
using People.Utils;


namespace People.Test {

	class OriginContactSource : Object, Backend.ContactSource,
			Backend.DumpableContactSource, Backend.QueryableContactSource {
		public weak People.Test.ProxyContactSource test {
			private get;
			private construct;
		}

		public Backend.ContactSourceFlags flags {
			get;
			protected set;
			default = Backend.ContactSourceFlags.NONE;
		}

		public OriginContactSource (People.Test.ProxyContactSource test) {
			this.test = test;
		}

		public AsyncOperation<Backend.Contact> request_contact(string? id) {
			test.request_contact_called = true;
			return new AsyncOperation<Backend.Contact> ();
		}

		public AsyncSearch<Backend.Contact> dump () {
			test.dump_called = true;
			return new AsyncSearch<Backend.Contact> ();
		}

		public AsyncSearch<Backend.Contact> query (Backend.Query? query) {
			test.query_called = true;
			return new AsyncSearch<Backend.Contact> ();
		}

		public void do_set_flags (Backend.ContactSourceFlags flags) {
			this.flags = flags;
		}
	}

	class ProxyContactSource : TestIt.Suite {
		private OriginContactSource origin;
		private Backend.ContactSource proxy;

		public bool request_contact_called;
		public bool dump_called;
		public bool query_called;

		protected override void initialize () {
			name = "People::ProxyContactSource";
			add_test_case ("flags", test_flags);
			add_test_case ("request contacts", test_request_contact);
			add_test_case ("dump", test_dump);
			add_test_case ("query", test_query);
		}

		protected override void set_up () {
			origin = new OriginContactSource (this);
			proxy = new People.ProxyContactSource (origin);

			request_contact_called = false;
			dump_called = false;
			query_called = false;
		}

		protected override void tear_down () {
			origin = null;
			proxy = null;
		}

		private void test_flags () {
			assert_true (proxy.flags == origin.flags);
			origin.do_set_flags (Backend.ContactSourceFlags.NEED_NETWORK);
			assert_true (proxy.flags == origin.flags);
		}

		private void test_request_contact () {
			proxy.request_contact (null);
			assert_true (request_contact_called);
		}

		private void test_dump () {
			((Backend.DumpableContactSource) proxy).dump ();
			assert_true (dump_called);
		}

		private void test_query () {
			((Backend.QueryableContactSource) proxy).query (null);
			assert_true (query_called);
		}

		static int main(string[] args) {
			var suite = new ProxyContactSource();
			return suite.run();
		}
	}
}
