using Gee;

using People;
using People.Utils;


namespace People.Test {

	class OriginContact : Backend.BaseContact {
		public weak People.Test.ProxyContact test {
			private get;
			private construct;
		}

		public OriginContact (People.Test.ProxyContact test) {
			this.test = test;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields() {
			test.get_defined_fields_called = true;
			return new AsyncOperation<Gee.Set<string> > ();
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields() {
			test.get_definable_fields_called = true;
			return new AsyncOperation<Gee.Set<string> > ();
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			test.get_mandatory_fields_called = true;
			return new AsyncOperation<Gee.Set<string> > ();
		}

		public override AsyncOperation<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			test.get_field_spec_called = true;
			return new AsyncOperation<Backend.ContactFieldSpec> ();
		}

		public override AsyncOperation<Gee.Map<string, string> > get_fields(Gee.Set<string> fields) {
			test.get_fields_called = true;
			return new AsyncOperation<Gee.Map<string, string> > ();
		}

		public override AsyncOperation<Gee.Set<string> > set_fields(Gee.Map<string, string> fields) {
			test.set_fields_called = true;
			return new AsyncOperation<Gee.Set<string> > ();
		}

		public override AsyncOperation<bool> @delete() {
			test.delete_called = true;
			return new AsyncOperation<bool> ();
		}

		public void do_set_flags (Backend.ContactFlags flags) {
			this.flags = flags;
		}

		public void do_set_id (string? id) {
			this.id = id;
		}
	}

	class ProxyContact : TestIt.Suite {
		private OriginContact origin;
		private Backend.Contact proxy;

		public bool get_defined_fields_called;
		public bool get_definable_fields_called;
		public bool get_mandatory_fields_called;
		public bool get_field_spec_called;
		public bool get_fields_called;
		public bool set_fields_called;
		public bool delete_called;

		protected override void initialize () {
			name = "People::ProxyContact";
			add_test_case ("flags", test_flags);
			add_test_case ("id", test_id);
			add_test_case ("get defined fields", test_get_defined_fields);
			add_test_case ("get definable fields", test_get_definable_fields);
			add_test_case ("get mandatory fields", test_get_mandatory_fields);
			add_test_case ("get field spec", test_get_field_spec);
			add_test_case ("get fields", test_get_field_spec);
			add_test_case ("set fields", test_set_fields);
			add_test_case ("delete", test_delete);
		}

		protected override void set_up () {
			origin = new OriginContact (this);
			proxy = new People.ProxyContact (origin);

			get_defined_fields_called = false;
			get_definable_fields_called = false;
			get_mandatory_fields_called = false;
			get_field_spec_called = false;
			get_fields_called = false;
			set_fields_called = false;
			delete_called = false;
		}

		protected override void tear_down () {
			origin = null;
			proxy = null;
		}

		private void test_flags () {
			assert_true (proxy.flags == origin.flags);
			origin.do_set_flags (Backend.ContactFlags.NONE);
			assert_true (proxy.flags == origin.flags);
		}

		private void test_id () {
			assert_true (proxy.id == origin.id);
			origin.do_set_id ("dummy");
			assert_true (proxy.id == origin.id);
		}

		private void test_get_defined_fields () {
			proxy.get_defined_fields ();
			assert_true (get_defined_fields_called);
		}

		private void test_get_definable_fields () {
			proxy.get_definable_fields ();
			assert_true (get_definable_fields_called);
		}

		private void test_get_mandatory_fields () {
			proxy.get_mandatory_fields ();
			assert_true (get_mandatory_fields_called);
		}

		private void test_get_field_spec () {
			proxy.get_field_spec ("");
			assert_true (get_field_spec_called);
		}

		private void test_set_fields () {
			proxy.set_fields (new HashMap<string, string> (str_hash, str_equal, str_equal));
			assert_true (set_fields_called);
		}

		private void test_get_fields () {
			proxy.get_fields (new HashSet<string> (str_hash, str_equal));
			assert_true (get_fields_called);
		}

		private void test_delete () {
			proxy.delete ();
			assert_true (delete_called);
		}

		static int main(string[] args) {
			var suite = new ProxyContact();
			return suite.run();
		}
	}
}
