using Gee;

using People.Utils;


namespace People.Test {

	class QueryBuilder : TestIt.Suite, Backend.QueryVisitor {
		private People.QueryBuilder builder;

		private uint nb_not;
		private uint nb_and;
		private uint nb_or;
		private uint nb_equals;
		private uint nb_contains;
		private uint nb_field;
		private uint nb_value;

		protected override void initialize () {
			name = "People::QueryBuilder";
			add_test_case ("correct query", test_correct_query);
			add_test_case ("malformed queries", test_malformed_query);
		}

		protected override void set_up () {
			builder = new People.QueryBuilder ();

			nb_not = 0;
			nb_and = 0;
			nb_or = 0;
			nb_equals = 0;
			nb_contains = 0;
			nb_field = 0;
			nb_value = 0;
		}

		protected override void tear_down () {
			builder = null;
		}

		private void test_correct_query () {
			var q =
			"(NOT" +
				"(OR" +
					"(AND (NOT (CONTAINS identity.name.full \"bob\"))" +
						"(EQUALS identity.birth.city communication.address.city))" +
					"(NOT (CONTAINS communication.phone 666)))" +
				")" +
			")";

			Backend.Query query = builder.build (q);
			query.accept (this);

			assert_true (nb_not == 3);
			assert_true (nb_and == 1);
			assert_true (nb_or == 1);
			assert_true (nb_equals == 1);
			assert_true (nb_contains == 2);
			assert_true (nb_field == 4);
			assert_true (nb_value == 2);
		}

		private void test_malformed_query () {
			var queries = new string[] {
				// "bad query",
				"(NOT)",
				"(NOT (EQUALS a b)",
				"(NOT (EQUALS a b c))",
				"(NOT (EQUALS a b) c)",
				"(AND (EQUALS a b)",
				"(OR (EQUALS a b)",
				"(CONTAINS a b c)",
				"(NOT (EQUALS a b",
				"(bla (EQUALS a d))",
				"(bla (EQUALS a d)))"
			};

			foreach (string query in queries) {
				var tree = builder.build (query);
				assert_true (tree == null);
			}
		}

		/* QueryVisitor interface */
		public void visit_composite (Backend.CompositeQuery query) {
			switch (query.operator) {
			case Backend.QueryOperator.NOT:
				nb_not++;
				break;
			case Backend.QueryOperator.AND:
				nb_and++;
				break;
			case Backend.QueryOperator.OR:
				nb_or++;
				break;
			case Backend.QueryOperator.EQUALS:
				nb_equals++;
				break;
			case Backend.QueryOperator.CONTAINS:
				nb_contains++;
				break;
			}

			foreach (Backend.Query q in query.queries) {
				q.accept (this);
			}
		}

		public void visit_field (Backend.FieldQuery query) {
			nb_field++;
		}

		public void visit_value (Backend.ValueQuery query) {
			nb_value++;
		}
		/* end of QueryVisitor interface */

		static int main(string[] args) {
			var suite = new QueryBuilder ();
			return suite.run ();
		}
	}
}
