using Gee;

using People;
using People.Utils;


namespace People.Test {

	private class DummyContact : Backend.BaseContact {
		private static Backend.ContactFieldSpec default_field_spec =
				new Backend.ContactFieldSpec(Backend.ContactFieldFlags.NONE);

		public Map<string, string> fields {
			private get;
			private construct set;
		}

		public DummyContact (string id, Map<string, string> fields) {
			this.id = id;
			this.fields = fields;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields() {
			return sync_result (fields.get_keys ());
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields() {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<Backend.ContactFieldSpec> get_field_spec(string field_name) {
			return sync_result (default_field_spec);
		}

		public override AsyncOperation<Gee.Map<string, string> >
				get_fields(Gee.Set<string> fields) {
			var result = new HashMap<string, string> (str_hash, str_equal, str_equal);
			foreach (string field in fields) {
				result.set (field, this.fields.get (field));
			}
			return sync_result (result);
		}

		public override AsyncOperation<Gee.Set<string> >
				set_fields(Gee.Map<string, string> fields) {
			return sync_result (new HashSet<string> (str_hash, str_equal));
		}

		public override AsyncOperation<bool> @delete() {
			return sync_result_bool (false);
		}
	}


	class QueryEvaluator : TestIt.Suite {

		Gee.Set<Backend.Contact> contacts;
		bool callback_called0;
		bool callback_called1;
		bool callback_called2;
		bool callback_called3;

		protected override void initialize () {
			name = "People::QueryEvaluator";
			add_test_case ("simple selection", test_simple_selection);
		}

		protected override void set_up () {
			contacts = new HashSet<Backend.Contact> ();

			var fields0 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields0.set ("identity.name.first", "Monsieur");
			fields0.set ("identity.name.last", "Baboul");
			fields0.set ("communication.email.home", "monsieur.baboul@live.com");
			contacts.add (new DummyContact ("0", fields0));

			var fields1 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields1.set ("identity.name.first", "Monsieur");
			fields1.set ("identity.name.last", "Baboul");
			fields1.set ("communication.email.home", "monsieur.baboul@gmail.com");
			contacts.add (new DummyContact ("1", fields1));

			var fields2 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields2.set ("identity.name.first", "Baboul");
			fields2.set ("identity.name.last", "Baboul");
			fields2.set ("communication.email.home", "monsieur.baboul@gmail.com");
			contacts.add (new DummyContact ("2", fields2));

			var fields3 = new HashMap<string, string> (str_hash, str_equal, str_equal);
			fields3.set ("identity.name.first", "Iorek");
			fields3.set ("identity.name.last", "Byrnison");
			fields3.set ("communication.email.home", "iorek@armoured-bear.fi");
			contacts.add (new DummyContact ("3", fields3));

			callback_called0 = false;
			callback_called1 = false;
			callback_called2 = false;
			callback_called3 = false;
		}

		protected override void tear_down () {
			contacts = null;
		}


		private void test_simple_selection () {
			var q0 = new Gee.ArrayList<Backend.Query> ();
			var q1 = new Gee.ArrayList<Backend.Query> ();
			var q2 = new Gee.ArrayList<Backend.Query> ();

			q1.add (new Backend.FieldQuery("identity.name.first"));
			q1.add (new Backend.FieldQuery("identity.name.last"));
			q0.add (Backend.Query.not (new Backend.CompositeQuery (Backend.QueryOperator.EQUALS, q1)));

			q2.add (new Backend.FieldQuery ("communication.email.home"));
			q2.add (new Backend.ValueQuery ("gmail.com"));
			q0.add (new Backend.CompositeQuery (Backend.QueryOperator.CONTAINS, q2));

			var query = Backend.Query.and (q0);

			/* (AND (NOT (EQUALS identity.name.first identity.name.last)
					(CONTAINS communication.email.home "gmail.com"))) */
			var helper = new People.QueryEvaluator (query);
			foreach (Backend.Contact contact in contacts) {
				var op = helper.evaluate (contact);
				op.add_callback (callback, contact);
			}

			assert_true (callback_called0);
			assert_true (callback_called1);
			assert_true (callback_called2);
			assert_true (callback_called3);
		}

		private void callback(AsyncOperation<bool> op, void* user_data) {
			var contact = (Backend.Contact) user_data;
			bool result = op.get_result ();

			if (contact.id == "0") {
				assert_true (!result);
				callback_called0 = true;
			} else if (contact.id == "1") {
				assert_true (result);
				callback_called1 = true;
			} else if (contact.id == "2") {
				assert_true (!result);
				callback_called2 = true;
			} else if (contact.id == "3") {
				assert_true (!result);
				callback_called3 = true;
			}
		}

		static int main (string[] args) {
			var suite = new QueryEvaluator ();
			return suite.run ();
		}
	}

}
